#include <linux/crypto.h>
#include <linux/scatterlist.h>

#define	DATA_SIZE  32

static int __init aes_user_init(void)
{
	char key[16], *iv;
	struct crypto_blkcipher *tfm_cbc;
	struct scatterlist sg[3];
	int ret;
	char *msg, *encrypted, *decrypted;
	struct blkcipher_desc desc;
	size_t ivsize;

	printk("aes-user\n");
	memset(key, 0, sizeof(key));

	tfm_cbc = crypto_alloc_blkcipher("cbc(aes)",0,CRYPTO_ALG_ASYNC);
	if ( IS_ERR(tfm_cbc) )
		return -EIO;

	desc.tfm = tfm_cbc;
	desc.flags = 0;

	ret = crypto_blkcipher_setkey(tfm_cbc, key, sizeof(key));
	if ( ret<0 )
		goto out;

	msg = kzalloc(DATA_SIZE, GFP_KERNEL);
	if ( !msg )
		goto out;

	encrypted = kzalloc(DATA_SIZE, GFP_KERNEL);
	if ( !encrypted )
		goto alloc_encrypted_failed;

	decrypted = kzalloc(DATA_SIZE, GFP_KERNEL);
	if ( !decrypted )
		goto alloc_decrypted_failed;

	memcpy(msg, "Linux Magazin", 13 );

	sg_init_table( sg, ARRAY_SIZE(sg) );
	sg_set_buf( &sg[0], msg, DATA_SIZE );
	sg_set_buf( &sg[1], encrypted, DATA_SIZE );
	sg_set_buf( &sg[2], decrypted, DATA_SIZE );

	iv = crypto_blkcipher_crt(tfm_cbc)->iv;
	ivsize = crypto_blkcipher_ivsize(tfm_cbc);
	memset(iv, 0, ivsize);

	ret = crypto_blkcipher_encrypt(&desc, &sg[1], &sg[0], DATA_SIZE);
	if ( ret<0 )
		goto crypt_failed;

	iv = crypto_blkcipher_crt(tfm_cbc)->iv;
	ivsize = crypto_blkcipher_ivsize(tfm_cbc);
	memset(iv, 0, ivsize);

	ret = crypto_blkcipher_decrypt(&desc, &sg[2], &sg[1], DATA_SIZE);
	if ( ret<0 )
		goto crypt_failed;

	print_hex_dump(KERN_INFO,"mesg> ",DUMP_PREFIX_NONE,16,1,msg,13,1);
	print_hex_dump(KERN_INFO,"encr> ",DUMP_PREFIX_NONE,16,1,encrypted,13,1);
	print_hex_dump(KERN_INFO,"decr> ",DUMP_PREFIX_NONE,16,1,decrypted,13,1);

crypt_failed:
	kfree(decrypted);
alloc_decrypted_failed:
	kfree(encrypted);
alloc_encrypted_failed:
	kfree(msg);
out:
	crypto_free_blkcipher(tfm_cbc);
	return -EIO;
}

static void __exit aes_user_exit(void)
{
}

module_init(aes_user_init);
module_exit(aes_user_exit);
MODULE_LICENSE("GPL");
